package com.liveness.h5.silentlivenessh5;

import static com.liveness.h5.h5liveness.LivenessH5Activity.KEY_EXTRA_IMAGE;
import static com.liveness.h5.h5liveness.LivenessH5Activity.KEY_EXTRA_RESULT;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.liveness.h5.h5liveness.LanguageEnum;
import com.liveness.h5.h5liveness.LivenessH5Activity;
import com.liveness.h5.h5liveness.RegionEnum;
import com.liveness.h5.h5liveness.util.Util;
import com.liveness.h5.silentlivenessh5.net.DFNetworkUtil;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;


public class MainActivity extends AppCompatActivity {
    private final static String TAG = MainActivity.class.getSimpleName();
    private ImageView imageView;
    private TextView textView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        imageView = findViewById(R.id.id_iv_result);
        textView = findViewById(R.id.id_tv_liveness);

        findViewById(R.id.btn_start).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent(MainActivity.this, LivenessH5Activity.class);
                Bundle bundle = new Bundle();
                bundle.putString(LivenessH5Activity.KEY_EXTRA_REGION, RegionEnum.INDIA.getCountry());
                bundle.putString(LivenessH5Activity.KEY_EXTRA_LANGUAGE, LanguageEnum.EN.getLan());
                intent.putExtras(bundle);
                startActivityForResult(intent, 100);
            }
        });

        Util.requestCameraPermission(this);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        Util.onRequestPermissionsResult(this, requestCode, permissions, grantResults);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        if (resultCode == RESULT_OK) {
            String imageId = data.getStringExtra(KEY_EXTRA_RESULT);
            byte[] imageJpeg = data.getByteArrayExtra(KEY_EXTRA_IMAGE);

            Bitmap bmp = BitmapFactory.decodeByteArray(imageJpeg, 0, imageJpeg.length);
            imageView.setImageBitmap(bmp);
            doLiveness(imageId);
        }

        super.onActivityResult(requestCode, resultCode, data);
    }

    private void doLiveness(final String imageId) {
        Future<DFNetworkUtil.DFNetResult> livenessFuture = Executors.newFixedThreadPool(1).submit(new Callable<DFNetworkUtil.DFNetResult>() {
            @Override
            public DFNetworkUtil.DFNetResult call() throws Exception {
                return DFNetworkUtil.doAntiHack(imageId);
            }
        });
        getNetworkResult(livenessFuture);
    }

    private void getNetworkResult(final Future<DFNetworkUtil.DFNetResult> livenessFuture) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    final DFNetworkUtil.DFNetResult result = livenessFuture.get();
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            String resultStr = "Liveness";
                            int color = R.color.green;
                            if (!result.mNetworkResultStatus) {
                                resultStr = "Hack";
                                color = R.color.red;
                            }
                            textView.setTextColor(getResources().getColor(color));
                            textView.setText(resultStr);
                        }
                    });
                } catch (ExecutionException e) {
                    e.printStackTrace();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }).start();
    }
}